Package: authordown
Title: Author Metadata Management and Manuscript Front Matter
Version: 0.1.0
Authors@R: c(
    person("Zhipeng", "Cao", email = "zhipeng30@foxmail.com", role = c("aut", "cre"))
  )
Author: Zhipeng Cao [aut, cre]
Maintainer: Zhipeng Cao <zhipeng30@foxmail.com>
Description: Manage author metadata and generate manuscript front matter.
    It produces title pages, acknowledgements, conflicts of interest, and
    contribution sections for large author lists, with helpers for validating
    and reading common spreadsheet formats.
License: AGPL-3
URL: https://zh1peng.github.io/authordown/,
        https://github.com/zh1peng/authordown
BugReports: https://github.com/zh1peng/authordown/issues
Encoding: UTF-8
Depends: R (>= 3.5)
RoxygenNote: 7.3.2
Imports: openxlsx, rlang
Suggests: knitr, pkgdown, rmarkdown, shiny, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-03-20 01:51:44 UTC; Zhipeng
Repository: CRAN
Date/Publication: 2026-03-24 09:20:08 UTC
Built: R 4.6.0; ; 2026-04-25 16:40:05 UTC; unix

# Generated by roxygen2: do not edit by hand

export(authordown)
export(authordown_read_local)
export(authordown_template)
export(authordown_validate)
export(generate_acknowledgement)
export(generate_conflict)
export(generate_contribution)
export(generate_template)
export(generate_title_page)
export(render_section_html)

# authordown

[![R-CMD-check](https://github.com/zh1peng/authordown/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/zh1peng/authordown/actions/workflows/R-CMD-check.yaml)

<img src="man/figures/logo.png" align="right" height="150" alt="authordown logo" />

authordown is an R package for managing author metadata and generating manuscript front matter, especially for large author lists.

 [中文文档](https://github.com/zh1peng/authordown/blob/master/README.zh.md)
 
Online app: [Open the app](https://zh1peng-authordown.share.connect.posit.cloud/)
## Features

- Title page generation with affiliation numbering and correspondence notes
- Author metadata validation and normalization
- Acknowledgements, conflict of interest, and contribution sections
- Templates to standardize author data entry
- Shiny app for quick preview and export

## Installation

From CRAN (once available):

```r
install.packages("authordown")
```

From GitHub:

```r
# install.packages("devtools")
devtools::install_github("zh1peng/authordown")
```

## End-to-end workflow (offline safe)

### 1) Start from the bundled template

```r
library(authordown)

# Create a fresh template in a temporary file
template_path <- authordown_template()
authors <- authordown_read_local(template_path)
```

### 2) Generate a title page (with degree shown)

```r
title_page <- generate_title_page(
  data = authors,
  title = "Example Paper",
  style = "default",
  show_degree = TRUE
)
cat(title_page)
```

### 3) Generate other sections

```r
ack <- generate_acknowledgement(authors, style = "paragraph")
coi <- generate_conflict(authors, style = "paragraph")
contrib <- generate_contribution(authors, style = "bullets")

cat(ack)
cat("\n\n")
cat(coi)
cat("\n\n")
cat(contrib)
```

### 4) XLSX input

```r
xlsx_path <- system.file("extdata", "authordown_template.xlsx", package = "authordown")
authors_xlsx <- authordown_read_local(xlsx_path)
```

## Recommended workflow for online tables

If you manage authors in an online table, export it locally and then use
`authordown_read_local()`:

1) Export to CSV or XLSX (or TSV).
2) Read locally with `authordown_read_local()`.

Supported formats: CSV, TSV, XLSX.

## Affiliations

Use `Affiliation1`, `Affiliation2`, ... `AffiliationN` columns to list all
affiliations for each author. There is no hard limit; add as many columns as
needed. The title page numbers affiliations in the order they first appear.

## Shiny app

Online app: [Open the app](https://zh1peng-authordown.share.connect.posit.cloud/)

```r
if (interactive()) {
  shiny::runApp(system.file("shiny-app", package = "authordown"))
}
```

Workflow: download the template, fill it, upload it, preview the parsed data, then generate outputs for copy/paste.

## Render sections for copy/paste

If you want rendered HTML for easy copy/paste into a manuscript system, use:

```r
if (requireNamespace("rmarkdown", quietly = TRUE) && rmarkdown::pandoc_available()) {
  template_path <- authordown_template()
  authors <- authordown_read_local(template_path)

  html_path <- render_section_html(
    section_title = "Conflict of Interest",
    content_function = generate_conflict,
    data = authors,
    style = "paragraph"
  )
  # utils::browseURL(html_path)
}
```

## Troubleshooting

- "Missing required column" means your file does not include required fields such as `FirstName` and `LastName`.
- "Corresponding authors must have an Email" means a row has `Corresponding = TRUE` but `Email` is blank.
- "ORCID values must use the 0000-0000-0000-0000 format" indicates a formatting error.

## License

AGPL-3

---
title: "Chinese README"
output:
  rmarkdown::html_vignette:
    self_contained: false
vignette: >
  %\VignetteIndexEntry{Chinese README}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, results='asis'}
# During `R CMD check`, vignette code is also run by sourcing a tangled `.R`
# script, where `knitr::current_input()` is not set.
path <- "README.zh.md"
if (!file.exists(path)) {
  path <- file.path("vignettes", "README.zh.md")
}
lines <- readLines(path, warn = FALSE)
lines <- lines[!grepl("R-CMD-check", lines)]
cat(lines, sep = "\n")
```

#' Render a Manuscript Section to HTML
#'
#' @description This function takes a section title, a content-generating function, and a data frame,
#' then produces an HTML file that displays the section header and its content. This generic function
#' works for any section (e.g. Conflict of Interest, Author Contributions, Acknowledgements).
#'
#' @param section_title A character string for the section header (e.g., "Conflict of Interest").
#' @param content_function A function that accepts a data frame and returns a formatted character string.
#' @param data A data frame containing the necessary columns.
#' @param output_file The path to the output HTML file. Defaults to a temporary file.
#' @param ... Additional arguments passed to \code{content_function}.
#' @return A character string with the path to the rendered HTML file.
#' @export
#' @examples
#' \donttest{
#' if (requireNamespace("rmarkdown", quietly = TRUE) && rmarkdown::pandoc_available()) {
#'   template_path <- authordown_template()
#'   authors <- authordown_read_local(template_path)
#'
#'   html_path <- render_section_html(
#'     section_title = "Conflict of Interest",
#'     content_function = generate_conflict,
#'     data = authors,
#'     style = "paragraph"
#'   )
#'   if (interactive()) utils::browseURL(html_path)
#' }
#' }
render_section_html <- function(section_title, content_function, data,
                                output_file = tempfile(fileext = ".html"), ...) {
  if (!requireNamespace("rmarkdown", quietly = TRUE)) {
    rlang::abort("Package 'rmarkdown' is required to render HTML output.")
  }
  # Get the section content
  content <- content_function(data, ...)
  
  # Build an R Markdown document with a section header and content
  rmd_content <- paste0(
    "---\n",
    "title: \"", "HTML output", "\"\n",
    "output: html_document\n",
    "---\n\n",
    "### ", section_title, "\n\n",
    content, "\n"
  )
  
  # Write the Rmd content to a temporary file and render it to HTML
  rmd_file <- tempfile(fileext = ".Rmd")
  writeLines(rmd_content, con = rmd_file)
  rmarkdown::render(input = rmd_file, output_file = output_file, quiet = TRUE)
  
  message("HTML file generated at: ", output_file)
  return(output_file)
}

#' Generate a sample CSV template for authordown
#'
#' @description Creates a sample CSV (or Excel) with columns for authors,
#' affiliations, acknowledgments, conflicts, etc. If `file` is omitted, a
#' temporary file is created.
#' @param file A file path where the template should be written. If omitted,
#'   a temporary file is created.
#' @param excel Logical. If TRUE, writes an Excel file (.xlsx) instead of CSV.
#' @return Invisibly returns the data frame used for the template.
#' @export
#' @examples
#' generate_template()
#' generate_template(excel = TRUE)
generate_template <- function(file = NULL, excel = FALSE) {
  format <- if (isTRUE(excel)) "xlsx" else "csv"
  if (is.null(file)) {
    file <- tempfile("authordown_template_", fileext = paste0(".", format))
  } else if (!grepl(paste0("\\.", format, "$"), file, ignore.case = TRUE)) {
    file <- paste0(file, ".", format)
  }

  authordown_template(path = file, format = format)
  message("Template written to: ", file)

  invisible(authordown_read_local(file, validate = FALSE))
}

#' Generate a Conflict of Interest Statement
#'
#' @description Produces a readable conflict statement. If all authors report no conflict,
#' it states so. Otherwise, it lists the authors reporting conflicts and then indicates that
#' the remaining authors declare no conflict.
#'
#' @param data A data frame containing at least the columns: FirstName, LastName, and Conflict.
#' @param style Output style: "paragraph", "bullets", or "numbered".
#' @return A character string with the formatted conflict statement.
#' @export
#' @examples
#' authors <- data.frame(
#'   FirstName = c("Alice", "Bob"),
#'   LastName = c("Smith", "Johnson"),
#'   Conflict = c("No conflict", "Consultant at Company Z"),
#'   stringsAsFactors = FALSE
#' )
#' generate_conflict(authors, style = "paragraph")
generate_conflict <- function(data, style = c("paragraph", "bullets", "numbered")) {
  style <- match.arg(style)
  if (!("Conflict" %in% names(data))) {
    return("No conflict of interest information provided.")
  }
  
  data$Conflict <- trimws(as.character(data$Conflict))
  
  conflict_authors <- data[!is.na(data$Conflict) & data$Conflict != "" &
                              tolower(data$Conflict) != "no conflict", ]
  non_conflict_authors <- data[is.na(data$Conflict) | data$Conflict == "" |
                                 tolower(data$Conflict) == "no conflict", ]
  
  if (nrow(conflict_authors) == 0) {
    return("All authors declare no conflict of interest.")
  }

  conflict_lines <- apply(conflict_authors, 1, function(row) {
    paste0(row["FirstName"], " ", row["LastName"], " (", row["Conflict"], ")")
  })

  if (style == "paragraph") {
    conflict_text <- paste(conflict_lines, collapse = "; ")
    result <- paste0(
      "The following authors report conflicts of interest: ",
      conflict_text, "."
    )
  } else if (style == "bullets") {
    conflict_text <- paste0("- ", conflict_lines, collapse = "\n")
    result <- paste("Conflicts of interest:", conflict_text, sep = "\n")
  } else {
    conflict_text <- paste0(seq_along(conflict_lines), ". ", conflict_lines, collapse = "\n")
    result <- paste("Conflicts of interest:", conflict_text, sep = "\n")
  }

  if (nrow(non_conflict_authors) > 0) {
    non_conflict_names <- apply(non_conflict_authors, 1, function(row) {
      paste(row["FirstName"], row["LastName"])
    })
    result <- paste0(
      result,
      " All other authors (",
      paste(non_conflict_names, collapse = ", "),
      ") declare no conflict of interest."
    )
  }
  result
}

## ----echo=FALSE, results='asis'-----------------------------------------------
# During `R CMD check`, vignette code is also run by sourcing a tangled `.R`
# script, where `knitr::current_input()` is not set.
path <- "README.zh.md"
if (!file.exists(path)) {
  path <- file.path("vignettes", "README.zh.md")
}
lines <- readLines(path, warn = FALSE)
lines <- lines[!grepl("R-CMD-check", lines)]
cat(lines, sep = "\n")

#' Generate all sections (title page, acknowledgement, conflict, contribution)
#'
#' @description A convenience wrapper that calls the various section generators
#' and combines them into one text block. Ideal for quick copy-paste into Word.
#'
#' @param data A data frame with the columns needed by each function:
#'   \code{generate_title_page}, \code{generate_acknowledgement},
#'   \code{generate_conflict}, \code{generate_contribution}.
#' @param title Optional paper title to pass to \code{generate_title_page()}.
#' @param style Title page style ("default", "APA", "Nature").
#' @param show_degree Logical. If TRUE, include Degree after author names.
#' @param acknowledgement_style Output style for acknowledgements.
#' @param conflict_style Output style for conflicts of interest.
#' @param contribution_style Output style for contributions.
#' @return A character string containing all sections.
#' @export
#' @examples
#' template_path <- authordown_template()
#' authors <- authordown_read_local(template_path)
#' cat(authordown(authors, title = "My Great Paper", style = "default"))
authordown <- function(data,
                       title = NULL,
                       style = "default",
                       show_degree = FALSE,
                       acknowledgement_style = "paragraph",
                       conflict_style = "paragraph",
                       contribution_style = "paragraph") {
  # 1) Title page
  tp <- generate_title_page(
    data = data,
    style = style,
    title = title,
    show_degree = show_degree,
    co_first_footnote = TRUE
  )

  # 2) Acknowledgements
  ack <- generate_acknowledgement(data, style = acknowledgement_style)

  # 3) Conflict of interest
  coi <- generate_conflict(data, style = conflict_style)

  # 4) Contributions
  contrib <- generate_contribution(data, style = contribution_style)

  # Combine all
  final_text <- paste(
    tp,
    "\n\n", ack,
    "\n\n", coi,
    "\n\n", contrib,
    sep = ""
  )

  return(final_text)
}

#' Generate Author Contributions
#'
#' @description Combines each author's contribution statement into a clear paragraph.
#' Each line indicates the author and their specific contribution.
#'
#' @param data A data frame containing at least the columns: FirstName, LastName, and Contribution.
#' @param style Output style: "paragraph", "bullets", or "numbered".
#' @param list_style Deprecated. Use \code{style}. When provided, TRUE maps to
#'   "bullets" and FALSE maps to "paragraph".
#' @return A character string summarizing the author contributions.
#' @export
#' @examples
#' authors <- data.frame(
#'   FirstName = c("Alice", "Bob"),
#'   LastName = c("Smith", "Johnson"),
#'   Contribution = c("Conceptualization; Data curation", "Supervision; Writing - review"),
#'   stringsAsFactors = FALSE
#' )
#' generate_contribution(authors, style = "paragraph")
generate_contribution <- function(data,
                                  style = c("paragraph", "bullets", "numbered"),
                                  list_style = NULL) {
  if (!is.null(list_style)) {
    if (!is.logical(list_style) || length(list_style) != 1) {
      rlang::abort("list_style must be TRUE or FALSE when provided.")
    }
    style <- if (isTRUE(list_style)) "bullets" else "paragraph"
  }
  style <- match.arg(style)
  if (!("Contribution" %in% names(data))) {
    return("No author contributions provided.")
  }
  
  valid <- data[!is.na(data$Contribution) & data$Contribution != "", ]
  if (nrow(valid) == 0) {
    return("No author contributions provided.")
  }
  
  lines <- apply(valid, 1, function(row) {
    paste0(
      row["FirstName"], " ", row["LastName"],
      " contributed as follows: ",
      row["Contribution"]
    )
  })

  if (style == "paragraph") {
    return(paste(lines, collapse = "; "))
  }
  if (style == "bullets") {
    return(paste0("- ", lines, collapse = "\n"))
  }

  paste0(seq_along(lines), ". ", lines, collapse = "\n")
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## -----------------------------------------------------------------------------
library(authordown)

csv_path <- system.file("extdata", "authordown_template.csv", package = "authordown")
authors <- authordown_read_local(csv_path)

## -----------------------------------------------------------------------------
csv_path <- authordown_template()
authors <- authordown_read_local(csv_path)
unlink(csv_path)

## -----------------------------------------------------------------------------
title_page <- generate_title_page(
  data = authors,
  title = "Example Paper",
  style = "default",
  show_degree = TRUE
)
cat(title_page)

## -----------------------------------------------------------------------------
ack <- generate_acknowledgement(authors, style = "paragraph")
coi <- generate_conflict(authors, style = "paragraph")
contrib <- generate_contribution(authors, style = "bullets")

cat(ack)
cat("\n\n")
cat(coi)
cat("\n\n")
cat(contrib)

## -----------------------------------------------------------------------------
xlsx_path <- system.file("extdata", "authordown_template.xlsx", package = "authordown")
authors_xlsx <- authordown_read_local(xlsx_path)

## ----eval=FALSE---------------------------------------------------------------
# html_path <- render_section_html(
#   section_title = "Conflict of Interest",
#   content_function = generate_conflict,
#   data = authors,
#   style = "paragraph"
# )

library(shiny)

load_authordown <- function() {
  r_dir <- file.path(getwd(), "R")
  if (!dir.exists(r_dir)) {
    stop("authordown sources not found in app directory.")
  }
  r_files <- list.files(r_dir, pattern = "\\.R$", full.names = TRUE)
  if (length(r_files) == 0) {
    stop("No R/*.R files found in app directory.")
  }
  for (r_file in r_files) {
    source(r_file, local = TRUE)
  }
  invisible(TRUE)
}

load_authordown()

server <- function(input, output, session) {
  read_input_data <- function(path, sheet = NULL) {
    if (!file.exists(path)) {
      rlang::abort(paste0("File not found: ", path))
    }
    ext <- tolower(tools::file_ext(path))
    if (ext %in% c("csv", "txt")) {
      data <- utils::read.csv(path, stringsAsFactors = FALSE, check.names = FALSE)
    } else if (ext == "tsv") {
      data <- utils::read.delim(path, stringsAsFactors = FALSE, check.names = FALSE)
    } else if (ext == "xlsx") {
      if (!requireNamespace("openxlsx", quietly = TRUE)) {
        rlang::abort("Package 'openxlsx' is required to read XLSX files.")
      }
      if (is.null(sheet)) {
        data <- openxlsx::read.xlsx(path)
      } else {
        data <- openxlsx::read.xlsx(path, sheet = sheet)
      }
    } else {
      rlang::abort("Unsupported file type. Use CSV, TSV, or XLSX.")
    }
    authordown_validate(data)
  }

  sample_data <- function() {
    sample_path <- system.file("extdata", "authordown_template.csv", package = "authordown")
    if (nzchar(sample_path)) {
      return(read_input_data(sample_path))
    }

    local_path <- file.path(getwd(), "extdata", "authordown_template.csv")
    if (file.exists(local_path)) {
      return(read_input_data(local_path))
    }

    repo_path <- file.path(getwd(), "inst", "extdata", "authordown_template.csv")
    if (file.exists(repo_path)) {
      return(read_input_data(repo_path))
    }

    stop("Sample template not found in package or repo.")
  }

  data_rv <- reactiveVal(sample_data())
  error_rv <- reactiveVal(NULL)

  observeEvent(input$author_file, {
    if (is.null(input$author_file)) {
      data_rv(sample_data())
      error_rv(NULL)
      return()
    }
    tryCatch({
      data_rv(read_input_data(input$author_file$datapath))
      error_rv(NULL)
    }, error = function(e) {
      error_rv(conditionMessage(e))
    })
  }, ignoreInit = TRUE)

  output$validation_msg <- renderText({
    msg <- error_rv()
    if (is.null(msg)) {
      "Validation: OK"
    } else {
      paste("Validation error:", msg)
    }
  })

  output$data_preview <- renderTable({
    data <- data_rv()
    validate(need(is.null(error_rv()), "Fix validation errors to preview data."))
    data
  }, rownames = TRUE)

  outputs_rv <- eventReactive(input$generate, {
    validate(need(is.null(error_rv()), "Fix validation errors before generating outputs."))
    data <- data_rv()

    title_page <- generate_title_page(
      data = data,
      title = input$paper_title,
      style = input$style,
      show_degree = input$show_degree
    )
    acknowledgements <- generate_acknowledgement(data, style = input$ack_style)
    conflict <- generate_conflict(data, style = input$coi_style)
    contributions <- generate_contribution(data, style = input$contrib_style)
    combined <- paste(title_page, acknowledgements, conflict, contributions, sep = "\n\n")

    list(
      title_page = title_page,
      acknowledgements = acknowledgements,
      conflict = conflict,
      contributions = contributions,
      combined = combined
    )
  })

  observeEvent(outputs_rv(), {
    updateTextAreaInput(session, "combined_preview", value = outputs_rv()$combined)
  })

  render_preview_html <- function(text) {
    if (is.null(text) || length(text) == 0) {
      return("")
    }
    lines <- strsplit(text, "\n", fixed = TRUE)[[1]]
    html_lines <- vapply(lines, function(line) {
      is_title <- grepl("^##\\s+", line)
      line_text <- sub("^##\\s+", "", line)
      line_text <- htmltools::htmlEscape(line_text)
      line_text <- gsub("\\^([^\\^]+)\\^([†*]?)", "<sup>\\1\\2</sup>", line_text, perl = TRUE)
      if (!grepl("^[†*]", line_text)) {
        line_text <- gsub("([A-Za-z0-9\\)])([†*])", "\\1<sup>\\2</sup>", line_text, perl = TRUE)
      }
      if (is_title) {
        line_text <- paste0("<strong>", line_text, "</strong>")
      }
      line_text
    }, character(1))

    html <- paste(html_lines, collapse = "<br>")
    htmltools::HTML(
      paste0(
        "<div style=\"white-space: normal; font-family: inherit; font-size: 14px;\">",
        html,
        "</div>"
      )
    )
  }

  output$combined_preview_rendered <- renderUI({
    req(outputs_rv())
    render_preview_html(outputs_rv()$combined)
  })

  output$titlepage <- renderUI({
    req(outputs_rv())
    render_preview_html(outputs_rv()$title_page)
  })

  output$acknowledgements <- renderUI({
    req(outputs_rv())
    render_preview_html(outputs_rv()$acknowledgements)
  })

  output$conflict <- renderUI({
    req(outputs_rv())
    render_preview_html(outputs_rv()$conflict)
  })

  output$contributions <- renderUI({
    req(outputs_rv())
    render_preview_html(outputs_rv()$contributions)
  })

  output$download_template_csv <- downloadHandler(
    filename = function() {
      "authordown_template.csv"
    },
    content = function(file) {
      authordown_template(path = file, format = "csv")
    }
  )

  output$download_template_xlsx <- downloadHandler(
    filename = function() {
      "authordown_template.xlsx"
    },
    content = function(file) {
      if (!requireNamespace("openxlsx", quietly = TRUE)) {
        stop("Package 'openxlsx' is required to generate XLSX templates.")
      }
      authordown_template(path = file, format = "xlsx")
    }
  )
}

#' Generate a Title Page (Journal-Style)
#'
#' @description Produces a journal-style title page text with:
#'   \enumerate{
#'     \item An optional paper title
#'     \item Author line (with superscript affiliation indices)
#'     \item Affiliation list (labeled by superscript numbers)
#'     \item Co-first author footnote (dagger symbol) if multiple authors share the same minimal rank
#'     \item Corresponding author footnote (*) and contact line
#'   }
#'
#' The function expects columns:
#'   \itemize{
#'     \item \strong{FirstName}, \strong{MiddleName}, \strong{LastName} (strings)
#'     \item \strong{Rank} (numeric), used to detect co-first authors (lowest rank)
#'     \item \strong{Correspondence} (logical or "TRUE"/"FALSE" string)
#'     \item \strong{Email} (for corresponding authors)
#'     \item \strong{Affiliation1}, \strong{Affiliation2}, etc. (any number of these)
#'   }
#'
#' @param data A data frame with the columns listed above.
#' @param style Title page style. Supported values: "default", "APA", "Nature".
#' @param title Optional character string for the paper title.
#' @param show_degree Logical. If TRUE, include Degree after author names.
#' @param co_first_footnote Logical. If \code{TRUE}, adds a footnote for co-first
#'   authors if multiple authors share the same \strong{lowest} rank.
#' @return A single character string suitable for copy-pasting into Word.
#' @export
#' @examples
#' authors <- data.frame(
#'   FirstName      = c("Alice", "Bob", "Charlie"),
#'   MiddleName     = c("M.", "", "Q."),
#'   LastName       = c("Smith", "Johnson", "Lee"),
#'   Degree         = c("PhD", "MD", "PhD"),
#'   Email          = c("alice@example.com", "bob@example.com", "charlie@example.org"),
#'   Rank           = c(1, 1, 2),  # Alice and Bob are co-first authors
#'   Correspondence = c(TRUE, FALSE, FALSE),
#'   Affiliation1   = c("University of X, Dept. of Y",
#'                      "University of X, Dept. of Y",
#'                      "Institute of Z"),
#'   Affiliation2   = c(NA, "Company W", NA),
#'   stringsAsFactors = FALSE
#' )
#' cat(generate_title_page(authors, title = "A Great Paper", show_degree = TRUE))
generate_title_page <- function(data,
                                style = c("default", "APA", "Nature"),
                                title = NULL,
                                show_degree = FALSE,
                                co_first_footnote = TRUE) {
  style <- match.arg(style)
  # 1) Sort authors by Rank if present
  if ("Rank" %in% colnames(data)) {
    data <- data[order(data$Rank, na.last = TRUE), ]
  }
  
  # 2) Identify co-first authors if multiple authors share the same minimal rank
  co_first_flag <- FALSE
  co_first_rank <- NULL
  if (co_first_footnote && "Rank" %in% colnames(data)) {
    if (!all(is.na(data$Rank))) {
      min_rank <- min(data$Rank, na.rm = TRUE)
      # Count how many authors have this rank
      if (sum(data$Rank == min_rank, na.rm = TRUE) > 1) {
        co_first_flag <- TRUE
        co_first_rank <- min_rank
      }
    }
  }
  
  # 3) Identify corresponding authors
  #    Accept "TRUE"/"FALSE" strings or logical
  if ("Correspondence" %in% colnames(data)) {
    is_corr <- function(x) {
      if (is.logical(x)) return(x)
      # If it's a string, check if it's "true", "TRUE", "yes", "1"
      tolower(trimws(x)) %in% c("true", "yes", "1")
    }
    data$IsCorresponding <- sapply(data$Correspondence, is_corr)
  } else if ("Corresponding" %in% colnames(data)) {
    is_corr <- function(x) {
      if (is.logical(x)) return(x)
      tolower(trimws(x)) %in% c("true", "yes", "1")
    }
    data$IsCorresponding <- sapply(data$Corresponding, is_corr)
  } else {
    data$IsCorresponding <- FALSE
  }
  
  # 4) Gather all affiliation columns
  aff_cols <- grep("^Affiliation", names(data), value = TRUE)
  # Extract unique non-NA affiliations in author order
  all_affils <- character()
  if (length(aff_cols) > 0) {
    for (row_idx in seq_len(nrow(data))) {
      for (col in aff_cols) {
        val <- as.character(data[row_idx, col])
        if (!is.na(val) && val != "" && !(val %in% all_affils)) {
          all_affils <- c(all_affils, val)
        }
      }
    }
  }
  # Indices for the unique affiliations
  aff_indices <- seq_along(all_affils)
  
  # 5) Build the author line
  #    e.g., "Alice M. Smith^1†*, Bob Johnson^1,2†, Charlie Q. Lee^3"
  author_strs <- apply(data, 1, function(row) {
    # Name
    nm_parts <- c(row["FirstName"], row["MiddleName"], row["LastName"])
    if (isTRUE(show_degree) && "Degree" %in% names(row)) {
      nm_parts <- c(nm_parts, row["Degree"])
    }
    nm_parts <- nm_parts[!is.na(nm_parts) & nm_parts != "" & nm_parts != "NA"]
    full_name <- paste(nm_parts, collapse = " ")
    
    # Affiliations for this author
    aff_for_author <- c()
    for (col in aff_cols) {
      val <- row[col]
      if (!is.na(val) && val != "") {
        # find index in all_affils
        idx <- which(all_affils == val)
        aff_for_author <- c(aff_for_author, idx)
      }
    }
    aff_superscript <- ""
    if (length(aff_for_author) > 0) {
      aff_for_author <- sort(unique(aff_for_author))
      aff_superscript <- paste0("^", paste(aff_for_author, collapse = ","), "^")
    }
    
    # Co-first footnote marker (†)
    # if rank == co_first_rank
    co_first_marker  <- ""
    if (co_first_flag && !is.na(row["Rank"]) && row["Rank"] == co_first_rank) {
      co_first_marker <- "\u2020"  # †
      # Add to the superscript if it exists
    }
    
    
    # Corresponding author marker (*)
    corr_marker <- ""
    if (as.logical(row["IsCorresponding"])) {
      corr_marker <- "*"
    }
    
    # e.g., "Alice M. Smith^1†*"
    paste0(full_name, aff_superscript, co_first_marker, corr_marker)
  })
  
  author_line <- paste(author_strs, collapse = ", ")
  
  # 6) Build the affiliation lines
  #    e.g.,
  #    "^1^ University of X, Dept. of Y
  #     ^2^ Company Z, Research Division"
  aff_lines <- mapply(function(i, aff) {
    paste0(i, ". ", aff)
  }, aff_indices, all_affils)
  aff_text <- paste(aff_lines, collapse = "\n\n")
  
  # 7) Build footnotes if needed
  footnotes <- c()
  
  # co-first note
  if (co_first_flag) {
    footnotes <- c(footnotes, "\u2020Co-first authors who contributed equally. \n\n")
  }
  
  # corresponding authors
  corr_idx <- which(data$IsCorresponding == TRUE)
  if (length(corr_idx) > 0 && "Email" %in% colnames(data)) {
    corr_entries <- vapply(corr_idx, function(i) {
      nm_parts <- c(data$FirstName[i], data$MiddleName[i], data$LastName[i])
      nm_parts <- nm_parts[!is.na(nm_parts) & nm_parts != "" & nm_parts != "NA"]
      full_name <- paste(nm_parts, collapse = " ")
      email <- as.character(data$Email[i])
      email <- email[!is.na(email) & email != ""]
      if (length(email) == 0) {
        return(full_name)
      }
      paste0(full_name, " <", email, ">")
    }, character(1))
    corr_entries <- unique(corr_entries[!is.na(corr_entries) & corr_entries != ""])
    if (length(corr_entries) > 0) {
      corr_line <- paste("Corresponding author(s):", paste(corr_entries, collapse = "; "))
      # Mark with '*'
      footnotes <- c(footnotes, paste0("*", corr_line))
    }
  }
  
  footnote_text <- ""
  if (length(footnotes) > 0) {
    footnote_text <- paste(footnotes, collapse = "\n")
  }
  
  # 8) Assemble final output
  # Something like:
  # Title: ...
  #
  # Alice M. Smith^1†*, Bob Johnson^1,2†
  #
  # ^1^ University of X, Dept. of Y
  # ^2^ Company Z, Research Division
  #
  # † indicates co-first authors...
  # * Corresponding author(s): ...
  
  lines <- c()
  
  if (!is.null(title) && title != "") {
    lines <- c(lines, paste0('## ',title, "\n"))
  }
  
  lines <- c(lines, author_line, "")
  if (length(all_affils) > 0) {
    lines <- c(lines, aff_text, "")
  }
  
  if (footnote_text != "") {
    lines <- c(lines, footnote_text)
  }
  
  final_output <- paste(lines, collapse = "\n")
  return(final_output)
}

#' Generate an Acknowledgement Section (Updated)
#'
#' @description Combines acknowledgements provided by the authors into a single,
#' coherent sentence. The function ignores author names (as they are assumed to be
#' included in the acknowledgement text itself) and simply concatenates each non-empty
#' acknowledgement entry using a semicolon.
#'
#' @param data A data frame containing at least the column Acknowledgement.
#' @param style Output style: "paragraph", "bullets", or "numbered".
#' @return A character string with the combined acknowledgements.
#' @export
#' @examples
#' authors <- data.frame(
#'   Acknowledgement = c("This work received support from resource X", 
#'                       "Resource Y was instrumental", 
#'                       "Alice Smith received support from resource ZZ"),
#'   stringsAsFactors = FALSE
#' )
#' generate_acknowledgement(authors, style = "paragraph")
generate_acknowledgement <- function(data,
                                     style = c("paragraph", "bullets", "numbered")) {
  style <- match.arg(style)
  if (!("Acknowledgement" %in% names(data))) {
    return("No acknowledgement information provided.")
  }
  
  valid <- data$Acknowledgement[!is.na(data$Acknowledgement) & data$Acknowledgement != ""]
  if (length(valid) == 0) {
    return("No acknowledgements provided.")
  }
  
  prefix <- "Acknowledgements:"
  if (style == "paragraph") {
    result <- paste(valid, collapse = "; ")
    return(paste(prefix, result))
  }
  if (style == "bullets") {
    bullets <- paste0("- ", valid, collapse = "\n")
    return(paste(prefix, bullets, sep = "\n"))
  }

  numbered <- paste0(seq_along(valid), ". ", valid, collapse = "\n")
  paste(prefix, numbered, sep = "\n")
}

#' Read a local author metadata file
#'
#' @description Reads a CSV/TSV/XLSX file and returns a standardized data frame.
#' By default, the data are validated for required columns and basic formats.
#'
#' @param path Path to a local CSV, TSV, or XLSX file.
#' @param sheet Optional sheet name or index for XLSX files.
#' @param validate Logical. If TRUE, validate the data.
#' @return A data frame with standardized columns.
#' @export
#' @examples
#' template_path <- authordown_template()
#' authors <- authordown_read_local(template_path)
authordown_read_local <- function(path, sheet = NULL, validate = TRUE) {
  if (!file.exists(path)) {
    rlang::abort(paste0("File not found: ", path))
  }

  ext <- tolower(tools::file_ext(path))
  if (ext %in% c("csv", "txt")) {
    data <- utils::read.csv(path, stringsAsFactors = FALSE, check.names = FALSE)
  } else if (ext == "tsv") {
    data <- utils::read.delim(path, stringsAsFactors = FALSE, check.names = FALSE)
  } else if (ext == "xlsx") {
    if (!requireNamespace("openxlsx", quietly = TRUE)) {
      rlang::abort("Package 'openxlsx' is required to read XLSX files.")
    }
    if (is.null(sheet)) {
      data <- openxlsx::read.xlsx(path)
    } else {
      data <- openxlsx::read.xlsx(path, sheet = sheet)
    }
  } else {
    rlang::abort("Unsupported file type. Use CSV, TSV, or XLSX.")
  }

  if (isTRUE(validate)) {
    return(authordown_validate(data))
  }

  authordown_canonicalize(data)
}

#' Write an authordown input template
#'
#' @description Copies a standard author metadata template from the package
#' into the requested path. Use this to start a new project with the expected
#' columns and example values. If `path` is omitted, a temporary file is used.
#'
#' @param path Output file path. If omitted, a temporary file is created.
#'   Extension determines format if \code{format} is not supplied.
#' @param format Optional format override: \code{"csv"} or \code{"xlsx"}.
#' @return Invisibly returns the output path.
#' @export
#' @examples
#' csv_path <- authordown_template()
#' xlsx_path <- authordown_template(format = "xlsx")
authordown_template <- function(path = NULL, format = NULL) {
  if (is.null(format)) {
    if (is.null(path)) {
      format <- "csv"
    } else {
      format <- tolower(tools::file_ext(path))
      if (format == "") {
        format <- "csv"
      }
    }
  }
  format <- match.arg(format, c("csv", "xlsx"))
  if (is.null(path)) {
    path <- tempfile("authordown_template_", fileext = paste0(".", format))
  }

  source_path <- system.file(
    "extdata",
    paste0("authordown_template.", format),
    package = "authordown"
  )
  if (source_path == "") {
    rlang::abort("Template file is missing from inst/extdata.")
  }

  ok <- file.copy(source_path, path, overwrite = TRUE)
  if (!ok) {
    rlang::abort(paste0("Failed to write template to: ", path))
  }

  invisible(path)
}

library(shiny)

fluidPage(
  titlePanel("authordown Manuscript Front Matter"),
  tags$p("Workflow: download template \u2192 upload completed file \u2192 preview data \u2192 generate outputs."),
  tags$hr(),

  tags$h3("1) Download template"),
  tags$p("Start by downloading the standard template and fill it in."),
  fluidRow(
    column(6, downloadButton("download_template_csv", "Download CSV template")),
    column(6, downloadButton("download_template_xlsx", "Download XLSX template"))
  ),

  tags$hr(),
  tags$h3("2) Upload completed template"),
  fileInput("author_file", "Upload CSV or XLSX", accept = c(".csv", ".xlsx", ".tsv")),
  tags$p("Supported formats: CSV, TSV, XLSX."),
  tags$p("Affiliations: use columns named Affiliation1, Affiliation2, ... AffiliationN."),
  verbatimTextOutput("validation_msg"),

  tags$hr(),
  tags$h3("3) Preview parsed data"),
  tags$div(
    style = "max-height: 320px; overflow-y: auto; border: 1px solid #ddd; padding: 6px;",
    tableOutput("data_preview")
  ),

  tags$hr(),
  tags$h3("4) Generate outputs"),
  fluidRow(
    column(4, selectInput("style", "Title page style", choices = c("default", "APA", "Nature"))),
    column(4, checkboxInput("show_degree", "Show degrees", value = FALSE)),
    column(4, textInput("paper_title", "Paper title", value = "A great paper"))
  ),
  fluidRow(
    column(4, selectInput("ack_style", "Acknowledgements style", choices = c("paragraph", "bullets", "numbered"))),
    column(4, selectInput("coi_style", "Conflict style", choices = c("paragraph", "bullets", "numbered"))),
    column(4, selectInput("contrib_style", "Contributions style", choices = c("paragraph", "bullets", "numbered")))
  ),
  actionButton("generate", "Generate outputs"),

  tags$hr(),
  tags$h3("5) Preview and copy"),
  tags$p("These previews are formatted for clean copy/paste."),
  uiOutput("combined_preview_rendered"),
  checkboxInput("show_plain", "Show plain text output", value = FALSE),
  conditionalPanel(
    condition = "input.show_plain",
    textAreaInput("combined_preview", "Combined output (plain text)", value = "", rows = 12)
  ),
  tabsetPanel(
    tabPanel("Title page", uiOutput("titlepage")),
    tabPanel("Acknowledgements", uiOutput("acknowledgements")),
    tabPanel("Conflict of interest", uiOutput("conflict")),
    tabPanel("Contributions", uiOutput("contributions"))
  )
)

#' Validate author metadata
#'
#' @description Checks required columns and key formats such as order, email,
#' ORCID, and corresponding author rules.
#'
#' @param data A data frame of author metadata.
#' @param require_affiliations Logical. If TRUE, require at least one
#'   affiliation column.
#' @return The validated, standardized data frame.
#' @export
#' @examples
#' authors <- data.frame(FirstName = "Alice", LastName = "Smith")
#' authordown_validate(authors)
authordown_validate <- function(data, require_affiliations = FALSE) {
  data <- authordown_canonicalize(data)
  problems <- attr(data, "authordown_problems")
  if (is.null(problems)) {
    problems <- list()
  }

  errors <- character()

  required_cols <- c("FirstName", "LastName")
  missing_cols <- setdiff(required_cols, names(data))
  if (length(missing_cols) > 0) {
    errors <- c(
      errors,
      paste0("Missing required column(s): ", paste(missing_cols, collapse = ", "))
    )
  }

  if (length(problems$Order) > 0) {
    errors <- c(errors, "Column 'Order' must be numeric integers.")
  }
  if ("Order" %in% names(data)) {
    if (any(!is.na(data$Order) & data$Order <= 0)) {
      errors <- c(errors, "Column 'Order' must be positive integers.")
    }
    order_vals <- data$Order[!is.na(data$Order)]
    if (any(duplicated(order_vals))) {
      errors <- c(errors, "Column 'Order' must be unique when provided.")
    }
  }

  if (length(problems$Corresponding) > 0) {
    errors <- c(errors, "Column 'Corresponding' must be TRUE/FALSE or yes/no.")
  }

  if ("Corresponding" %in% names(data) && "Email" %in% names(data)) {
    needs_email <- !is.na(data$Corresponding) & data$Corresponding
    missing_email <- needs_email & (is.na(data$Email) | data$Email == "")
    if (any(missing_email)) {
      errors <- c(errors, "Corresponding authors must have an Email.")
    }
  }

  if ("Email" %in% names(data)) {
    email_vals <- trimws(as.character(data$Email))
    data$Email <- email_vals
    bad_idx <- which(!is.na(email_vals) &
      !grepl("^[^@[:space:]]+@[^@[:space:]]+$", email_vals))
    if (length(bad_idx) > 0) {
      shown <- paste0(
        "row ", bad_idx, ": ", shQuote(email_vals[bad_idx])
      )
      shown <- paste(utils::head(shown, 5), collapse = ", ")
      if (length(bad_idx) > 5) {
        shown <- paste0(shown, ", ...")
      }
      errors <- c(
        errors,
        paste0("Email values must look like valid addresses (", shown, ").")
      )
    }
  }

  if ("Orcid" %in% names(data)) {
    orcid_vals <- data$Orcid[!is.na(data$Orcid)]
    bad_orcid <- orcid_vals[
      !grepl("^[0-9]{4}-[0-9]{4}-[0-9]{4}-[0-9]{3}[0-9X]$", orcid_vals)
    ]
    if (length(bad_orcid) > 0) {
      errors <- c(errors, "ORCID values must use the 0000-0000-0000-0000 format.")
    }
  }
  aff_cols <- grep("^Affiliation\\d+$", names(data), value = TRUE)
  if (isTRUE(require_affiliations) && length(aff_cols) == 0) {
    errors <- c(errors, "At least one Affiliation column is required.")
  }

  if (length(errors) > 0) {
    rlang::abort(paste(c("Invalid author data:", errors), collapse = "\n"))
  }

  data
}

#' Standardize raw author metadata
#'
#' @param data A data frame of author metadata.
#' @return A standardized data frame.
#' @keywords internal
authordown_canonicalize <- function(data) {
  if (!is.data.frame(data)) {
    rlang::abort("Input must be a data frame.")
  }

  data <- as.data.frame(data, stringsAsFactors = FALSE, check.names = FALSE)
  names(data) <- trimws(names(data))

  normalized <- function(x) {
    gsub("[^a-z0-9]+", "", tolower(x))
  }

  target_map <- list(
    FirstName = c("firstname", "givenname"),
    MiddleName = c("middlename"),
    LastName = c("lastname", "surname", "familyname"),
    Suffix = c("suffix"),
    Degree = c("degree"),
    Email = c("email", "emailaddress"),
    Orcid = c("orcid"),
    Order = c("order", "authororder"),
    Rank = c("rank"),
    Corresponding = c("corresponding", "correspondence", "correspondingauthor"),
    Contribution = c("contribution", "credit"),
    Acknowledgement = c("acknowledgement", "acknowledgment", "acknowledgments"),
    Conflict = c("conflict", "coi", "conflictofinterest")
  )

  new_names <- names(data)
  for (i in seq_along(new_names)) {
    raw_name <- new_names[i]
    norm <- normalized(raw_name)
    if (grepl("^affiliation\\d+$", tolower(raw_name))) {
      new_names[i] <- paste0(
        "Affiliation",
        gsub("^affiliation", "", tolower(raw_name))
      )
      next
    }
    if (norm == "affiliation") {
      new_names[i] <- "Affiliation1"
      next
    }
    matched <- FALSE
    for (target in names(target_map)) {
      if (norm %in% target_map[[target]]) {
        new_names[i] <- target
        matched <- TRUE
        break
      }
    }
    if (!matched) {
      new_names[i] <- raw_name
    }
  }
  names(data) <- new_names

  if (any(duplicated(names(data)))) {
    rlang::abort("Duplicate column names after standardization.")
  }

  for (col in names(data)) {
    if (is.factor(data[[col]])) {
      data[[col]] <- as.character(data[[col]])
    }
    if (is.character(data[[col]])) {
      data[[col]] <- trimws(data[[col]])
      data[[col]][data[[col]] == ""] <- NA
    }
  }

  problems <- list()

  if ("Corresponding" %in% names(data)) {
    raw_vals <- data$Corresponding
    parsed <- vapply(raw_vals, parse_logical_value, logical(1))
    invalid <- which(!is.na(raw_vals) & is.na(parsed))
    if (length(invalid) > 0) {
      problems$Corresponding <- invalid
    }
    data$Corresponding <- parsed
  }

  if ("Order" %in% names(data)) {
    raw_vals <- data$Order
    parsed <- suppressWarnings(as.integer(as.character(raw_vals)))
    invalid <- which(!is.na(raw_vals) & is.na(parsed))
    if (length(invalid) > 0) {
      problems$Order <- invalid
    }
    data$Order <- parsed
  }
  if ("Orcid" %in% names(data)) {
    data$Orcid <- toupper(data$Orcid)
  }

  attr(data, "authordown_problems") <- problems
  data
}

parse_logical_value <- function(x) {
  if (is.logical(x)) {
    return(x)
  }
  if (is.na(x)) {
    return(NA)
  }
  x <- tolower(trimws(as.character(x)))
  if (x %in% c("true", "t", "yes", "y", "1")) {
    return(TRUE)
  }
  if (x %in% c("false", "f", "no", "n", "0")) {
    return(FALSE)
  }
  NA
}

---
title: "End-to-End Workflow"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{End-to-End Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Overview

authordown standardizes author metadata and produces common manuscript sections. This vignette uses bundled example data so it can run offline and deterministically.

## Load example data

```{r}
library(authordown)

csv_path <- system.file("extdata", "authordown_template.csv", package = "authordown")
authors <- authordown_read_local(csv_path)
```

If you want a fresh template in a temporary file:

```{r}
csv_path <- authordown_template()
authors <- authordown_read_local(csv_path)
unlink(csv_path)
```

## Title page

```{r}
title_page <- generate_title_page(
  data = authors,
  title = "Example Paper",
  style = "default",
  show_degree = TRUE
)
cat(title_page)
```

## Acknowledgements, conflict, and contributions

```{r}
ack <- generate_acknowledgement(authors, style = "paragraph")
coi <- generate_conflict(authors, style = "paragraph")
contrib <- generate_contribution(authors, style = "bullets")

cat(ack)
cat("\n\n")
cat(coi)
cat("\n\n")
cat(contrib)
```

## XLSX input

```{r}
xlsx_path <- system.file("extdata", "authordown_template.xlsx", package = "authordown")
authors_xlsx <- authordown_read_local(xlsx_path)
```

## Online tables (export to local file first)

If you manage authors in an online table, export it locally and read the file.

1) Export to CSV or XLSX (or TSV).
2) Read locally with `authordown_read_local()`.

Supported formats: CSV, TSV, XLSX.

## Affiliations

Use `Affiliation1`, `Affiliation2`, ... `AffiliationN` columns to list all
affiliations for each author. There is no hard limit; add as many columns as
needed. The title page numbers affiliations in the order they first appear.

## Troubleshooting

- Missing required columns means your input does not include required fields such as `FirstName` and `LastName`.
- Corresponding authors must have an Email means a row has `Corresponding = TRUE` but `Email` is blank.
- ORCID values must match `0000-0000-0000-0000` format.

## Render sections for copy/paste

If you want HTML for clean copy/paste into a manuscript system, use `render_section_html()`:

```{r, eval=FALSE}
html_path <- render_section_html(
  section_title = "Conflict of Interest",
  content_function = generate_conflict,
  data = authors,
  style = "paragraph"
)
```
